import java.util.NoSuchElementException;
//@interface DatoNaVezbama {}

// Tip podataka koji sadrzi binarno stablo
class BinarnoStablo<T> {

	// Tip podataka koji opisuje jedan cvor stabla
	private static class Cvor<T> {
		
		T element;
		Cvor<T> levo;
		Cvor<T> desno;

		Cvor(T element, Cvor<T> levo, Cvor<T> desno) {
			this.element = element;
			this.levo = levo;
			this.desno = desno;
		}
	}

	// Cvor koji je koren stabla
	// Ako je stablo prazno, koren je null
	protected Cvor<T> koren;

	// Kreiramo prazno stablo
	public BinarnoStablo() {
		koren = null;
	}

	// Kreiramo stablo sa datim elementom u korenu
	// bez levog i desnog podstabla
	public BinarnoStablo(T element) {
		koren = new Cvor<>(element, null, null);
	}

	// Kreiramo novo stablo sa datim elementom u korenu
	// i datim levim i desnim podstablom
	public BinarnoStablo(T element, BinarnoStablo<T> levo, BinarnoStablo<T> desno) {
		koren = new Cvor<T>(element, levo.koren, desno.koren);
	}

	// Zasticeni konstruktor za kreiranje novog stabla
	// sa korenom u datom cvoru ovog stabla
	protected BinarnoStablo(Cvor<T> koren) {
		this.koren = koren;
	}

	public T getElement() {
		if (koren == null) { // Stablo je prazno
			throw new NoSuchElementException();
		}
		return koren.element;
	}

	public void setElement(T element) {
		if (koren == null) { // Stablo je prazno
			throw new NoSuchElementException();
		}
		koren.element = element;
	}

	public BinarnoStablo<T> getLevoPodstablo() {
		if (koren == null) { // Stablo je prazno
			throw new NoSuchElementException();
		}
		if (koren.levo == null) { // Nema levog podstabla
			return null;
		}
		return new BinarnoStablo<T>(koren.levo);
	}

	public void setLevoPodstablo(BinarnoStablo<T> levo) {
		if (koren == null) { // Stablo je prazno
			throw new NoSuchElementException();
		}
		if (levo == null) {
			koren.levo = null;
		} else {
			koren.levo = levo.koren;
		}
	}

	public BinarnoStablo<T> getDesnoPodstablo() {
		if (koren == null) { // Stablo je prazno
			throw new NoSuchElementException();
		}
		if (koren.desno == null) {
			return null;
		}
		return new BinarnoStablo<T>(koren.desno);
	}

	public void setDesnoPodstablo(BinarnoStablo<T> desno) {
		if (koren == null) { // Stablo je prazno
			throw new NoSuchElementException();
		}
		if (desno == null) {
			koren.desno = null;
		} else {
			koren.desno = desno.koren;
		}
	}

//    @DatoNaVezbama
	public int brojElemenata() {
    	// Spremimo argumente i pozovemo rekurzivni metod
    	return brojElemenata(koren);
	}

  //  @DatoNaVezbama
    // Metod koji zapravo resava dati problem
    protected static <T> int brojElemenata(Cvor<T> cvor) {
    	if (cvor == null) {
    		return 0;
    	}
		int broj = 1;
		broj = broj + brojElemenata(cvor.levo);
		broj = broj + brojElemenata(cvor.desno);
		return broj;
    }

 //   @DatoNaVezbama
	public int brojNivoa() {
    	// Spremimo argumente i pozovemo rekurzivni metod
    	return brojNivoa(koren);
	}

   @DatoNaVezbama
    // Metod koji zapravo resava dati problem
	protected static int brojNivoa(Cvor cvor) {
    	if (cvor == null) {
    		return 0;
    	}
		int brojNivoaLevog = brojNivoa(cvor.levo);
		int brojNivoaDesnog = brojNivoa(cvor.desno);
		return Math.max(brojNivoaLevog, brojNivoaDesnog) + 1;
	}
}

// Glavna klasa
public class StabloProgram {

	// Glavni program
	public static void main(String[] args) {

		// Kreiramo objekat koji koristimo za ucitavanje i ispisivanje stabla
		StabloIOPretty<Osoba> io = new StabloIOPretty<>(Konverter.OSOBA);

		// Ucitamo stablo
		BinarnoStablo<Osoba> stablo = io.readStablo(Svetovid.in("Osobe.txt"));

		// Ispisemo ucitano stablo na ekran
		io.printStablo(Svetovid.out, stablo);
		Svetovid.out.println();

		// Ilustrujemo poziv metoda
		
		int brojElemenata = stablo.brojElemenata();
		Svetovid.out.println("Broj elemenata:", brojElemenata);
		
		int brojNivoa = stablo.brojNivoa();
		Svetovid.out.println("Broj nivoa:", brojNivoa);

		// TODO Dodati pozive ostalih metoda

	}
}
